/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.metron.dataloads.nonbulk.flatfile.location;

import java.io.*;
import java.util.List;
import java.util.Optional;
import java.util.zip.GZIPInputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

public interface RawLocation<T> {
  Optional<List<String>> list(String loc) throws IOException;
  boolean exists(String loc) throws IOException;
  boolean isDirectory(String loc) throws IOException;

  InputStream openInputStream(String loc) throws IOException;
  boolean match(String loc);
  default void init(T state) {

  }

  default BufferedReader openReader(String loc) throws IOException {
    InputStream is = openInputStream(loc);
    if(loc.endsWith(".gz")) {
      return new BufferedReader(new InputStreamReader(new GZIPInputStream(is)));
    }
    else if(loc.endsWith(".zip")) {
      ZipInputStream zis = new ZipInputStream(is);
      ZipEntry entry = zis.getNextEntry();
      if(entry != null) {
        return new BufferedReader(new InputStreamReader(zis));
      }
      else {
        return new BufferedReader(new InputStreamReader(new ByteArrayInputStream(new byte[] {})));
      }
    }
    else {
      return new BufferedReader(new InputStreamReader(is));
    }
  }
}
